function varargout = dwdm(varargin)
% 
% This program is used to set difference wallpapers for the dual-monitor
% system.
%
%
% The first modification was on Monday of Dec. 28, 2009.
% By Limin Shao on Monday of Sept. 28, 2009.

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @dwdm_OpeningFcn, ...
                   'gui_OutputFcn',  @dwdm_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before dwdm is made visible.
function dwdm_OpeningFcn(hObject, eventdata, handles, varargin)
handles.output = hObject;

guidata(hObject, handles);

MonitorPosition = get(0, 'MonitorPositions'); 
% It should be noted that the Matlab 'Help' on this property is not 
% completely correct.  'MonitorPositions' returns coordinates corresponding 
% to the upper left corner and the bottom right corner (Windows convention).
if size(MonitorPosition, 1) < 2
    warndlg('Found only one monitor.', 'Warning');
    set([handles.Button_PrimaryPicture ...
        handles.Button_SecondaryPicture ...
        handles.Button_Go], 'Enable', 'Off');
    return;
end

Resolution = MonitorPosition(:, 3: 4) - MonitorPosition(:, 1: 2) + 1;
% Monitor resolution in the width-by-height form.  The first line is for 
% the primary monitor, and the second is for the secondary one.  

PrimaryFrameSize = [Resolution(1, 2) Resolution(1, 1)];
SecondaryFrameSize = [Resolution(2, 2) Resolution(2, 1)];
set(gcf, 'UserData', [PrimaryFrameSize; SecondaryFrameSize]);

set([handles.Text_SecondaryMonitor_Up, handles.Text_SecondaryMonitor_Down, ...
    handles.Text_SecondaryMonitor_Left], 'BackgroundColor', [0.1 0.1 0.4], ...
    'String', '', 'Enable', 'Off');
set(handles.Text_PrimaryMonitor, 'UserData', 'LR');
% This is the default layout of the dual-monitor system.
return;


function varargout = dwdm_OutputFcn(hObject, eventdata, handles) 
varargout{1} = handles.output;


function Text_SecondaryMonitor_Up_ButtonDownFcn(hObject, eventdata, handles)
set(hObject, 'BackgroundColor', [0 0 1], 'ForegroundColor', [0 1 0], 'String', 'Secondary', 'Enable', 'On');
set(handles.Text_PrimaryMonitor, 'UserData', 'DU');
set([handles.Text_SecondaryMonitor_Left, handles.Text_SecondaryMonitor_Right, ...
    handles.Text_SecondaryMonitor_Down], 'BackgroundColor', [0.1 0.1 0.4], ...
    'String', '', 'Enable', 'Off');


function Text_SecondaryMonitor_Left_ButtonDownFcn(hObject, eventdata, handles)
set(hObject, 'BackgroundColor', [0 0 1], 'ForegroundColor', [0 1 0], 'String', 'Secondary', 'Enable', 'On');
set(handles.Text_PrimaryMonitor, 'UserData', 'RL');
set([handles.Text_SecondaryMonitor_Up, handles.Text_SecondaryMonitor_Down, ...
    handles.Text_SecondaryMonitor_Right], 'BackgroundColor', [0.1 0.1 0.4], ...
    'String', '', 'Enable', 'Off');


function Text_SecondaryMonitor_Down_ButtonDownFcn(hObject, eventdata, handles)
set(hObject, 'BackgroundColor', [0 0 1], 'ForegroundColor', [0 1 0], 'String', 'Secondary', 'Enable', 'On');
set(handles.Text_PrimaryMonitor, 'UserData', 'UD');
set([handles.Text_SecondaryMonitor_Left, handles.Text_SecondaryMonitor_Right, ...
    handles.Text_SecondaryMonitor_Up], 'BackgroundColor', [0.1 0.1 0.4], ...
    'String', '', 'Enable', 'Off');


function Text_SecondaryMonitor_Right_ButtonDownFcn(hObject, eventdata, handles)
set(hObject, 'BackgroundColor', [0 0 1], 'ForegroundColor', [0 1 0], 'String', 'Secondary', 'Enable', 'On');
set(handles.Text_PrimaryMonitor, 'UserData', 'LR');
set([handles.Text_SecondaryMonitor_Up, handles.Text_SecondaryMonitor_Down, ...
    handles.Text_SecondaryMonitor_Left], 'BackgroundColor', [0.1 0.1 0.4], ...
    'String', '', 'Enable', 'Off');


function Text_Color_ButtonDownFcn(hObject, eventdata, handles)
Color = uisetcolor([0 0 0]);
set(hObject, 'UserData', Color, 'BackgroundColor', Color);


function Button_PrimaryPicture_Callback(hObject, eventdata, handles)
set(hObject, 'UserData', []);
[FileName, PathName, FilterIndex] = uigetfile('*.*', 'Load a picture for the primary monitor');

if FilterIndex == 0
    return; % Nothing loaded.
end

try
    PrimaryPicture = imread([PathName FileName]);
    set(hObject, 'TooltipString', [PathName FileName], 'UserData', PrimaryPicture);
    temp = get(gcf, 'UserData');
    PrimaryFrameSize = temp(1, :);
    if [size(PrimaryPicture, 1) size(PrimaryPicture, 2)] < PrimaryFrameSize
        set(handles.Popupmenu_PrimaryPictureMode, 'String', ...
            [get(handles.Popupmenu_PrimaryPictureMode, 'String'); 'Center']);
    end
catch
    errordlg(['Fail to load the picture: ' PathName FileName], 'Error');
end


function Button_SecondaryPicture_Callback(hObject, eventdata, handles)
set(hObject, 'UserData', []);
[FileName, PathName, FilterIndex] = uigetfile('*.*', 'Load a picture for the secondary monitor');

if FilterIndex == 0
    return; % Nothing loaded.
end

try
    SecondaryPicture = imread([PathName FileName]);
    set(hObject, 'TooltipString', [PathName FileName], 'UserData', SecondaryPicture);
    temp = get(gcf, 'UserData');
    SecondaryFrameSize = temp(2, :);
    if [size(SecondaryPicture, 1) size(SecondaryPicture, 2)] < SecondaryFrameSize
        set(handles.Popupmenu_SecondaryPictureMode, 'String', ...
            [get(handles.Popupmenu_SecondaryPictureMode, 'String'); 'Center']);
    end
catch
    errordlg(['Fail to load the picture: ' PathName FileName], 'Error');
end


function Popupmenu_PrimaryPictureMode_CreateFcn(hObject, eventdata, handles)
Control_Initialization(hObject, handles);


function Popupmenu_SecondaryPictureMode_CreateFcn(hObject, eventdata, handles)
Control_Initialization(hObject, handles);


function Button_Go_Callback(hObject, eventdata, handles)
PictureMode = {'tile' 'stretch' 'center'};
Color = get(handles.Text_Color, 'UserData') * 255;

temp = get(gcf, 'UserData');
PrimaryFrameSize = temp(1, :); SecondaryFrameSize = temp(2, :);
PrimaryFrame = ones(PrimaryFrameSize(1), PrimaryFrameSize(2), 3);
SecondaryFrame = ones(SecondaryFrameSize(1), SecondaryFrameSize(2), 3);
for iColor = 1:3, PrimaryFrame(:, :, iColor) = PrimaryFrame(:, :, iColor)*Color(iColor); end;
for iColor = 1:3, SecondaryFrame(:, :, iColor) = SecondaryFrame(:, :, iColor)*Color(iColor); end;
% Preset the primary and the secondary frame and the size inforamtion.

PrimaryPicture = get(handles.Button_PrimaryPicture, 'UserData');
SecondaryPicture = get(handles.Button_SecondaryPicture, 'UserData');
if isempty(PrimaryPicture) || isempty(SecondaryPicture)
    return;
end
PrimaryPictureSize = [size(PrimaryPicture, 1) size(PrimaryPicture, 2)];
SecondaryPictureSize = [size(SecondaryPicture, 1) size(SecondaryPicture, 2)];
% Get the primary and the secondary picture and the size information.

PrimaryPictureMode = PictureMode{get(handles.Popupmenu_PrimaryPictureMode, 'Value')};
SecondaryPictureMode = PictureMode{get(handles.Popupmenu_SecondaryPictureMode, 'Value')};

switch lower(PrimaryPictureMode)
    case 'center'
        Index = ceil((PrimaryFrameSize - PrimaryPictureSize) / 2);
        PrimaryFrame(Index(1):PrimaryPictureSize(1) + Index(1) - 1, ...
            Index(2):PrimaryPictureSize(2) + Index(2) - 1, :) = PrimaryPicture;
    otherwise
        Index1 = fitpicture(PrimaryPictureMode, PrimaryPictureSize, PrimaryFrameSize, 1);
        Index2 = fitpicture(PrimaryPictureMode, PrimaryPictureSize, PrimaryFrameSize, 2);
        PrimaryFrame = PrimaryPicture(Index1, Index2, :);
end
switch lower(SecondaryPictureMode)
    case 'center'
        Index = ceil((SecondaryFrameSize - SecondaryPictureSize) / 2);
        SecondaryFrame(Index(1):SecondaryPictureSize(1) + Index(1) - 1, ...
            Index(2): SecondaryPictureSize(2) + Index(2) - 1, :) = SecondaryPicture;
    otherwise
        Index1 = fitpicture(SecondaryPictureMode, SecondaryPictureSize, SecondaryFrameSize, 1);
        Index2 = fitpicture(SecondaryPictureMode, SecondaryPictureSize, SecondaryFrameSize, 2);
        SecondaryFrame = SecondaryPicture(Index1, Index2, :);
end
% Get the primary and the secondary frame with pictures fit in.

MonitorLayout = get(handles.Text_PrimaryMonitor, 'UserData'); 
switch lower(MonitorLayout)
    case {'lr', 'rl'}
        if size(PrimaryFrame, 1) > size(SecondaryFrame, 1)
            Patch = ones(size(PrimaryFrame, 1) - size(SecondaryFrame, 1), size(SecondaryFrame, 2), 3, 'uint8');
            for iColor = 1:3, Patch(:, :, iColor) = Patch(:, :, iColor)*Color(iColor); end;
            SecondaryFrame = [SecondaryFrame; Patch];
        else
            Patch = ones(size(SecondaryFrame, 1) - size(PrimaryFrame, 1), size(PrimaryFrame, 2), 3, 'uint8');
            for iColor = 1:3, Patch(:, :, iColor) = Patch(:, :, iColor)*Color(iColor); end;
            PrimaryFrame = [PrimaryFrame; Patch];
        end
    case {'du', 'ud'}
        if size(PrimaryFrame, 2) > size(SecondaryFrame, 2)
            Patch = ones(size(SecondaryFrame, 1), size(PrimaryFrame, 2) - size(SecondaryFrame, 2), 3, 'uint8');
            for iColor = 1:3, Patch(:, :, iColor) = Patch(:, :, iColor)*Color(iColor); end;
            SecondaryFrame = [SecondaryFrame Patch];
        else
            Patch = ones(size(PrimaryFrame, 1), size(SecondaryFrame, 2) - size(PrimaryFrame, 2), 3, 'uint8');
            for iColor = 1:3, Patch(:, :, iColor) = Patch(:, :, iColor)*Color(iColor); end;
            PrimaryFrame = [PrimaryFrame Patch];
        end
    otherwise
        % give the warning, and quit.
end
% Patch one of the frame so that they match each other according to the
% monitor layout.

switch lower(MonitorLayout)
    case 'lr'
        imwrite([PrimaryFrame SecondaryFrame], 'dwdm.bmp', 'bmp');
    case 'rl'
        imwrite([SecondaryFrame PrimaryFrame], 'dwdm.bmp', 'bmp');
    case 'du'
        imwrite([SecondaryFrame; PrimaryFrame], 'dwdm.bmp', 'bmp');
    case 'ud'
        imwrite([PrimaryFrame; SecondaryFrame], 'dwdm.bmp', 'bmp');
    otherwise
        % give the warning.
end
% stitch the two frames together according to the monitor layout.


function Control_Initialization(hObject, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function PixelIndex = fitpicture(Mode, PictureSize, FrameSize, Dimension)
switch lower(Mode)
    case 'tile'
        AuxMatrix = ones(1, ceil(FrameSize(Dimension) / PictureSize(Dimension)));
        AuxVector = (1: PictureSize(Dimension))' * AuxMatrix;
        PixelIndex = AuxVector(1: FrameSize(Dimension));
    case 'stretch'
        PixelIndex = round((0:FrameSize(Dimension)-1) / (FrameSize(Dimension) - 1) * (PictureSize(Dimension) - 1) + 1);
    otherwise
        % give the warning.
end
